use crate::auth::Auth;
use crate::models::{Form, LoginCredentials, LoginResponse, Submission};
use crate::AppState;
use actix_web::{web, Error as ActixWebError, HttpResponse, Responder, Result as ActixResult};
use chrono; // Only import the module since we use it qualified
use log;
use regex::Regex; // For pattern validation
use rusqlite::{params, Connection};
use serde_json::{json, Map, Value as JsonValue}; // Alias for clarity
use std::collections::HashMap;
use std::sync::{Arc, Mutex};
use uuid::Uuid;

// --- Helper Function for Validation ---

/// Validates submission data against the form field definitions with enhanced checks.
///
/// Expected field definition properties:
/// - `name`: string (required)
/// - `type`: string (e.g., "string", "number", "boolean", "email", "url", "object", "array") (required)
/// - `required`: boolean (optional, default: false)
/// - `maxLength`: number (for "string" type)
/// - `minLength`: number (for "string" type)
/// - `min`: number (for "number" type)
/// - `max`: number (for "number" type)
/// - `pattern`: string (regex for "string", "email", "url" types)
///
/// Returns `Ok(())` if valid, or `Err(JsonValue)` containing validation errors.
fn validate_submission_against_definition(
    submission_data: &JsonValue,
    form_definition_fields: &JsonValue,
) -> Result<(), JsonValue> {
    let mut errors: HashMap<String, String> = HashMap::new();

    // Ensure 'fields' in the definition is a JSON array
    let field_definitions = match form_definition_fields.as_array() {
        Some(defs) => defs,
        None => {
            log::error!(
                "Form definition 'fields' is not a JSON array. Def: {:?}",
                form_definition_fields
            );
            errors.insert(
                "_internal".to_string(),
                "Invalid form definition format (not an array)".to_string(),
            );
            return Err(json!({ "validation_errors": errors }));
        }
    };

    // Ensure the submission data is a JSON object
    let data_map = match submission_data.as_object() {
        Some(map) => map,
        None => {
            errors.insert(
                "_submission".to_string(),
                "Submission data must be a JSON object".to_string(),
            );
            return Err(json!({ "validation_errors": errors }));
        }
    };

    // Build a map of valid field names to their definitions from the definition for quick lookup
    let defined_field_names: HashMap<String, &Map<String, JsonValue>> = field_definitions
        .iter()
        .filter_map(|val| val.as_object())
        .filter_map(|def| {
            def.get("name")
                .and_then(JsonValue::as_str)
                .map(|name| (name.to_string(), def))
        })
        .collect();

    // 1. Check for submitted fields that are NOT in the definition
    for submitted_key in data_map.keys() {
        if !defined_field_names.contains_key(submitted_key) {
            errors.insert(
                submitted_key.clone(),
                "Unexpected field submitted".to_string(),
            );
        }
    }
    // Exit early if unexpected fields were found
    if !errors.is_empty() {
        log::warn!("Submission validation failed: Unexpected fields submitted.");
        return Err(json!({ "validation_errors": errors }));
    }

    // 2. Iterate through each field definition and validate corresponding submitted data
    for (field_name, field_def) in &defined_field_names {
        // Extract properties using helper functions for clarity
        let field_type = field_def
            .get("type")
            .and_then(JsonValue::as_str)
            .unwrap_or("string"); // Default to "string" if type is missing or not a string
        let is_required = field_def
            .get("required")
            .and_then(JsonValue::as_bool)
            .unwrap_or(false); // Default to false if required is missing or not a boolean
        let min_length = field_def.get("minLength").and_then(JsonValue::as_u64);
        let max_length = field_def.get("maxLength").and_then(JsonValue::as_u64);
        let min_value = field_def.get("min").and_then(JsonValue::as_f64); // Use f64 for flexibility
        let max_value = field_def.get("max").and_then(JsonValue::as_f64);
        let pattern = field_def.get("pattern").and_then(JsonValue::as_str);

        match data_map.get(field_name) {
            Some(submitted_value) if !submitted_value.is_null() => {
                // Field is present and not null, perform type and constraint checks
                let mut type_error = None;
                let mut constraint_errors = vec![];

                match field_type {
                    "string" | "email" | "url" => {
                        if let Some(s) = submitted_value.as_str() {
                            if let Some(min) = min_length {
                                if (s.chars().count() as u64) < min {
                                    // Use chars().count() for UTF-8 correctness
                                    constraint_errors
                                        .push(format!("Must be at least {} characters long", min));
                                }
                            }
                            if let Some(max) = max_length {
                                if (s.chars().count() as u64) > max {
                                    constraint_errors.push(format!(
                                        "Must be no more than {} characters long",
                                        max
                                    ));
                                }
                            }
                            if let Some(pat) = pattern {
                                // Consider caching compiled Regex if performance is critical
                                // and patterns are reused frequently across requests.
                                match Regex::new(pat) {
                                    Ok(re) => {
                                        if !re.is_match(s) {
                                           constraint_errors.push(format!("Does not match required pattern"));
                                        }
                                    }
                                    Err(e) => log::warn!("Invalid regex pattern '{}' in form definition for field '{}': {}", pat, field_name, e), // Log regex compilation error
                                }
                            }
                            // Specific checks for email/url
                            if field_type == "email" {
                                // Basic email regex (adjust for stricter needs or use a validation crate)
                                // This regex is very basic and allows many technically invalid addresses.
                                // Consider crates like `validator` for more robust validation.
                                let email_regex =
                                    Regex::new(r"^[^\s@]+@[^\s@]+\.[^\s@]+$").unwrap(); // Safe unwrap for known valid regex
                                if !email_regex.is_match(s) {
                                    constraint_errors
                                        .push("Must be a valid email address".to_string());
                                }
                            }
                            if field_type == "url" {
                                // Basic URL check (consider `url` crate for robustness)
                                if url::Url::parse(s).is_err() {
                                    constraint_errors.push("Must be a valid URL".to_string());
                                }
                            }
                        } else {
                            type_error = Some(format!("Expected a string for '{}'", field_name));
                        }
                    }
                    "number" => {
                        // Use as_f64 for flexibility (handles integers and floats)
                        if let Some(num) = submitted_value.as_f64() {
                            if let Some(min) = min_value {
                                if num < min {
                                    constraint_errors.push(format!("Must be at least {}", min));
                                }
                            }
                            if let Some(max) = max_value {
                                if num > max {
                                    constraint_errors.push(format!("Must be no more than {}", max));
                                }
                            }
                        } else {
                            type_error = Some(format!("Expected a number for '{}'", field_name));
                        }
                    }
                    "boolean" => {
                        if !submitted_value.is_boolean() {
                            type_error = Some(format!(
                                "Expected a boolean (true/false) for '{}'",
                                field_name
                            ));
                        }
                    }
                    "object" => {
                        if !submitted_value.is_object() {
                            type_error =
                                Some(format!("Expected a JSON object for '{}'", field_name));
                        }
                        // TODO: Could add deeper validation for object structure here if needed based on definition
                    }
                    "array" => {
                        if !submitted_value.is_array() {
                            type_error =
                                Some(format!("Expected a JSON array for '{}'", field_name));
                        }
                        // TODO: Could add validation for array elements here if needed based on definition
                    }
                    _ => {
                        // Log unsupported types during development/debugging if necessary
                        log::trace!(
                            "Unsupported field type '{}' encountered during validation for field '{}'. Treating as valid.",
                            field_type,
                            field_name
                        );
                        // Assume valid if type is not specifically handled or unknown
                    }
                }

                // Record errors found for this field
                if let Some(err) = type_error {
                    errors.insert(field_name.clone(), err);
                } else if !constraint_errors.is_empty() {
                    // Combine multiple constraint errors if necessary
                    errors.insert(field_name.clone(), constraint_errors.join("; "));
                }
            } // End check for present and non-null value
            Some(_) => {
                // Value is present but explicitly null (e.g., "fieldName": null)
                if is_required {
                    errors.insert(
                        field_name.clone(),
                        "This field is required and cannot be null".to_string(),
                    );
                }
                // Otherwise, null is considered a valid (empty) value for non-required fields
            }
            None => {
                // Field is missing entirely from the submission object
                if is_required {
                    errors.insert(field_name.clone(), "This field is required".to_string());
                }
                // Missing is valid for non-required fields
            }
        } // End match data_map.get(field_name)
    } // End loop through field definitions

    // Check if any errors were collected
    if errors.is_empty() {
        Ok(()) // Validation passed
    } else {
        log::info!(
            "Submission validation failed with {} error(s).", // Log only the count for brevity
            errors.len()
        );
        // Return a JSON object containing the specific validation errors
        Err(json!({ "validation_errors": errors }))
    }
}

// Helper function to convert anyhow::Error to actix_web::Error
fn anyhow_to_actix_error(e: anyhow::Error) -> ActixWebError {
    actix_web::error::ErrorInternalServerError(e.to_string())
}

// --- Public Handlers ---

// POST /login
pub async fn login(
    app_state: web::Data<AppState>, // Expect AppState like other handlers
    creds: web::Json<LoginCredentials>,
) -> ActixResult<impl Responder> {
    // Clone the Arc<Mutex<Connection>> from AppState
    let db_conn_arc = app_state.db.clone();
    let username = creds.username.clone();
    let password = creds.password.clone();

    // Wrap the blocking database operations in web::block
    let auth_result = web::block(move || {
        // Use the cloned Arc here
        let conn = db_conn_arc
            .lock()
            .map_err(|_| anyhow::anyhow!("Database mutex poisoned during login lock"))?;
        crate::db::authenticate_user(&conn, &username, &password)
    })
    .await
    .map_err(|e| {
        log::error!("web::block error during authentication: {:?}", e);
        actix_web::error::ErrorInternalServerError("Authentication process failed (blocking error)")
    })?
    .map_err(anyhow_to_actix_error)?;

    match auth_result {
        Some(user_data) => {
            // Clone Arc again for token generation, using the AppState db field
            let db_conn_token_arc = app_state.db.clone();
            let user_id = user_data.id.clone();

            // Generate and store a new token within web::block
            let token = web::block(move || {
                // Use the cloned Arc here
                let conn = db_conn_token_arc
                    .lock()
                    .map_err(|_| anyhow::anyhow!("Database mutex poisoned during token lock"))?;
                crate::db::generate_and_set_token_for_user(&conn, &user_id)
            })
            .await
            .map_err(|e| {
                log::error!("web::block error during token generation: {:?}", e);
                actix_web::error::ErrorInternalServerError(
                    "Failed to complete login (token generation blocking error)",
                )
            })?
            .map_err(anyhow_to_actix_error)?;

            log::info!("Login successful for user_id: {}", user_data.id);
            Ok(HttpResponse::Ok().json(LoginResponse { token }))
        }
        None => {
            log::warn!("Login failed for username: {}", creds.username);
            // Return 401 Unauthorized for failed login attempts
            Err(actix_web::error::ErrorUnauthorized(
                "Invalid username or password",
            ))
        }
    }
}

// POST /logout
pub async fn logout(
    app_state: web::Data<AppState>, // Expect AppState
    auth: Auth,                     // Requires authentication (extracts user_id from token)
) -> ActixResult<impl Responder> {
    log::info!("User {} requesting logout", auth.user_id);
    let db_conn_arc = app_state.db.clone(); // Get db from AppState
    let user_id = auth.user_id.clone();

    // Invalidate the token in the database within web::block
    web::block(move || {
        let conn = db_conn_arc // Use the cloned Arc
            .lock()
            .map_err(|_| anyhow::anyhow!("Database mutex poisoned during logout lock"))?;
        crate::db::invalidate_token(&conn, &user_id)
    })
    .await
    .map_err(|e| {
        // Use the original auth.user_id here as user_id moved into the block
        log::error!(
            "web::block error during logout for user {}: {:?}",
            auth.user_id,
            e
        );
        actix_web::error::ErrorInternalServerError("Logout failed (blocking error)")
    })?
    .map_err(anyhow_to_actix_error)?;

    log::info!("User {} logged out successfully", auth.user_id);
    Ok(HttpResponse::Ok().json(json!({ "message": "Logged out successfully" })))
}

// POST /forms/{form_id}/submissions
pub async fn submit_form(
    app_state: web::Data<AppState>,
    path: web::Path<String>,                  // Extracts form_id from path
    submission_payload: web::Json<JsonValue>, // Expect arbitrary JSON payload
) -> ActixResult<impl Responder> {
    let form_id = path.into_inner();
    let conn = app_state.db.lock().map_err(|e| {
        log::error!("Failed to acquire database lock: {}", e);
        actix_web::error::ErrorInternalServerError("Database error")
    })?;

    // Get form definition
    let form = Form::get_by_id(&conn, &form_id).map_err(anyhow_to_actix_error)?;

    // Validate submission against form definition
    if let Err(validation_errors) =
        validate_submission_against_definition(&submission_payload, &form.fields)
    {
        return Ok(HttpResponse::BadRequest().json(validation_errors));
    }

    // Create submission record
    let submission = Submission {
        id: Uuid::new_v4().to_string(),
        form_id: form_id.clone(),
        data: submission_payload.into_inner(),
        created_at: chrono::Utc::now(),
    };

    // Save submission to database
    submission.save(&conn).map_err(|e| {
        log::error!("Failed to save submission: {}", e);
        actix_web::error::ErrorInternalServerError("Failed to save submission")
    })?;

    // Send notifications if configured
    if let Some(notify_email) = form.notify_email {
        let email_subject = format!("New submission for form: {}", form.name);
        let email_body = format!(
            "A new submission has been received for form '{}'.\n\nSubmission ID: {}\nTimestamp: {}\n\nData:\n{}",
            form.name,
            submission.id,
            submission.created_at,
            serde_json::to_string_pretty(&submission.data).unwrap_or_default()
        );

        if let Err(e) = app_state
            .notification_service
            .send_email(&notify_email, &email_subject, &email_body)
            .await
        {
            log::warn!("Failed to send email notification: {}", e);
        }

        // Also send ntfy notification if configured (sends to the global topic)
        if let Some(topic_flag) = &form.notify_ntfy_topic {
            // Use field presence as a flag
            if !topic_flag.is_empty() {
                // Check if the flag string is non-empty
                let ntfy_title = format!("New submission for: {}", form.name);
                let ntfy_message = format!("Form: {}\nSubmission ID: {}", form.name, submission.id);
                if let Err(e) = app_state.notification_service.send_ntfy(
                    &ntfy_title,
                    &ntfy_message,
                    Some(3), // Medium priority
                ) {
                    log::warn!("Failed to send ntfy notification (global topic): {}", e);
                }
            }
        }
    }

    Ok(HttpResponse::Created().json(json!({
        "message": "Submission received",
        "submission_id": submission.id
    })))
}

// POST /forms
pub async fn create_form(
    app_state: web::Data<AppState>,
    _auth: Auth, // Authentication check via Auth extractor
    payload: web::Json<serde_json::Value>,
) -> ActixResult<impl Responder> {
    let payload = payload.into_inner();

    // Extract form data from payload
    let name = payload["name"]
        .as_str()
        .ok_or_else(|| actix_web::error::ErrorBadRequest("Missing or invalid 'name' field"))?
        .to_string();

    let fields = payload["fields"].clone();
    if !fields.is_array() {
        return Err(actix_web::error::ErrorBadRequest(
            "'fields' must be a JSON array",
        ));
    }

    let notify_email = payload["notify_email"].as_str().map(|s| s.to_string());
    let notify_ntfy_topic = payload["notify_ntfy_topic"].as_str().map(|s| s.to_string());

    // Create new form
    let form = Form {
        id: None, // Will be generated during save
        name,
        fields,
        notify_email,
        notify_ntfy_topic,
        created_at: chrono::Utc::now(),
    };

    // Save the form
    let conn = app_state.db.lock().map_err(|e| {
        log::error!("Failed to acquire database lock: {}", e);
        actix_web::error::ErrorInternalServerError("Database error")
    })?;

    form.save(&conn).map_err(|e| {
        log::error!("Failed to save form: {}", e);
        actix_web::error::ErrorInternalServerError("Failed to save form")
    })?;

    Ok(HttpResponse::Created().json(form))
}

// GET /forms
pub async fn get_forms(
    app_state: web::Data<AppState>,
    auth: Auth, // Requires authentication
) -> ActixResult<impl Responder> {
    log::info!("User {} requesting list of forms", auth.user_id);

    let conn = app_state.db.lock().map_err(|e| {
        log::error!("Failed to acquire database lock: {}", e);
        actix_web::error::ErrorInternalServerError("Database error")
    })?;

    let mut stmt = conn
        .prepare("SELECT id, name, fields, notify_email, notify_ntfy_topic, created_at FROM forms")
        .map_err(|e| {
            log::error!("Failed to prepare statement: {}", e);
            actix_web::error::ErrorInternalServerError("Database error")
        })?;

    let forms_iter = stmt
        .query_map([], |row| {
            let id: String = row.get(0)?;
            let name: String = row.get(1)?;
            let fields_str: String = row.get(2)?;
            let notify_email: Option<String> = row.get(3)?;
            let notify_ntfy_topic: Option<String> = row.get(4)?;
            let created_at: chrono::DateTime<chrono::Utc> = row.get(5)?;

            // Parse the 'fields' JSON string
            let fields: serde_json::Value = serde_json::from_str(&fields_str).map_err(|e| {
                log::error!(
                    "DB Parse Error: Failed to parse 'fields' JSON for form id {}: {}. Skipping this form.",
                    id,
                    e
                );
                rusqlite::Error::FromSqlConversionFailure(
                    2,
                    rusqlite::types::Type::Text,
                    Box::new(e),
                )
            })?;

            Ok(Form {
                id: Some(id),
                name,
                fields,
                notify_email,
                notify_ntfy_topic,
                created_at,
            })
        })
        .map_err(|e| {
            log::error!("Failed to execute query: {}", e);
            actix_web::error::ErrorInternalServerError("Database error")
        })?;

    // Collect results, filtering out rows that failed parsing
    let forms: Vec<Form> = forms_iter
        .filter_map(|result| match result {
            Ok(form) => Some(form),
            Err(e) => {
                log::warn!("Skipping a form row due to a processing error: {}", e);
                None
            }
        })
        .collect();

    log::debug!("Returning {} forms for user {}", forms.len(), auth.user_id);
    Ok(HttpResponse::Ok().json(forms))
}

// GET /forms/{form_id}/submissions
pub async fn get_submissions(
    app_state: web::Data<AppState>,
    auth: Auth,              // Requires authentication
    path: web::Path<String>, // Extracts form_id from the path
) -> ActixResult<impl Responder> {
    let form_id = path.into_inner();
    log::info!(
        "User {} requesting submissions for form_id: {}",
        auth.user_id,
        form_id
    );

    let conn = app_state.db.lock().map_err(|e| {
        log::error!("Failed to acquire database lock: {}", e);
        actix_web::error::ErrorInternalServerError("Database error")
    })?;

    // Check if the form exists
    let _form = Form::get_by_id(&conn, &form_id).map_err(|e| {
        if e.to_string().contains("not found") {
            actix_web::error::ErrorNotFound("Form not found")
        } else {
            actix_web::error::ErrorInternalServerError("Database error")
        }
    })?;

    // Get submissions
    let mut stmt = conn
        .prepare(
            "SELECT id, form_id, data, created_at FROM submissions WHERE form_id = ?1 ORDER BY created_at DESC",
        )
        .map_err(|e| {
            log::error!("Failed to prepare statement: {}", e);
            actix_web::error::ErrorInternalServerError("Database error")
        })?;

    let submissions_iter = stmt
        .query_map(params![form_id], |row| {
            let id: String = row.get(0)?;
            let form_id: String = row.get(1)?;
            let data_str: String = row.get(2)?;
            let created_at: chrono::DateTime<chrono::Utc> = row.get(3)?;

            let data: serde_json::Value = serde_json::from_str(&data_str).map_err(|e| {
                log::error!(
                "DB Parse Error: Failed to parse 'data' JSON for submission_id {}: {}. Skipping.",
                id,
                e
            );
                rusqlite::Error::FromSqlConversionFailure(
                    2,
                    rusqlite::types::Type::Text,
                    Box::new(e),
                )
            })?;

            Ok(Submission {
                id,
                form_id,
                data,
                created_at,
            })
        })
        .map_err(|e| {
            log::error!("Failed to execute query: {}", e);
            actix_web::error::ErrorInternalServerError("Database error")
        })?;

    let submissions: Vec<Submission> = submissions_iter
        .filter_map(|result| match result {
            Ok(submission) => Some(submission),
            Err(e) => {
                log::warn!("Skipping a submission row due to processing error: {}", e);
                None
            }
        })
        .collect();

    log::debug!(
        "Returning {} submissions for form {} requested by user {}",
        submissions.len(),
        form_id,
        auth.user_id
    );
    Ok(HttpResponse::Ok().json(submissions))
}

// --- Notification Settings Handlers ---

// GET /forms/{form_id}/notifications
pub async fn get_notification_settings(
    app_state: web::Data<AppState>,
    auth: Auth, // Requires authentication
    path: web::Path<String>,
) -> ActixResult<impl Responder> {
    let form_id = path.into_inner();
    log::info!(
        "User {} requesting notification settings for form_id: {}",
        auth.user_id,
        form_id
    );

    let conn = app_state.db.lock().map_err(|e| {
        log::error!(
            "Failed to acquire database lock for get_notification_settings: {}",
            e
        );
        actix_web::error::ErrorInternalServerError("Database error")
    })?;

    // Get the form to ensure it exists and retrieve current settings
    let form = Form::get_by_id(&conn, &form_id).map_err(|e| {
        log::warn!(
            "Attempt to get settings for non-existent form {}: {}",
            form_id,
            e
        );
        if e.to_string().contains("not found") {
            actix_web::error::ErrorNotFound("Form not found")
        } else {
            actix_web::error::ErrorInternalServerError("Database error retrieving form")
        }
    })?;

    let settings = crate::models::NotificationSettingsPayload {
        notify_email: form.notify_email,
        notify_ntfy_topic: form.notify_ntfy_topic,
    };

    Ok(HttpResponse::Ok().json(settings))
}

// PUT /forms/{form_id}/notifications
pub async fn update_notification_settings(
    app_state: web::Data<AppState>,
    auth: Auth, // Requires authentication
    path: web::Path<String>,
    payload: web::Json<crate::models::NotificationSettingsPayload>,
) -> ActixResult<impl Responder> {
    let form_id = path.into_inner();
    let new_settings = payload.into_inner();
    log::info!(
        "User {} updating notification settings for form_id: {}. Settings: {:?}",
        auth.user_id,
        form_id,
        new_settings
    );

    let conn = app_state.db.lock().map_err(|e| {
        log::error!(
            "Failed to acquire database lock for update_notification_settings: {}",
            e
        );
        actix_web::error::ErrorInternalServerError("Database error")
    })?;

    // Fetch the existing form to update it
    let mut form = Form::get_by_id(&conn, &form_id).map_err(|e| {
        log::warn!(
            "Attempt to update settings for non-existent form {}: {}",
            form_id,
            e
        );
        if e.to_string().contains("not found") {
            actix_web::error::ErrorNotFound("Form not found")
        } else {
            actix_web::error::ErrorInternalServerError("Database error retrieving form")
        }
    })?;

    // Update the form fields
    form.notify_email = new_settings.notify_email;
    form.notify_ntfy_topic = new_settings.notify_ntfy_topic;

    // Save the updated form
    form.save(&conn).map_err(|e| {
        log::error!(
            "Failed to save updated notification settings for form {}: {}",
            form_id,
            e
        );
        actix_web::error::ErrorInternalServerError("Failed to save notification settings")
    })?;

    log::info!(
        "Successfully updated notification settings for form {}",
        form_id
    );
    Ok(HttpResponse::Ok().json(json!({ "message": "Notification settings updated successfully" })))
}

pub async fn health_check() -> impl Responder {
    HttpResponse::Ok().json(serde_json::json!({
        "status": "ok",
        "version": env!("CARGO_PKG_VERSION"),
        "timestamp": chrono::Utc::now().to_rfc3339()
    }))
}
